import { AbstractControl, FormArray, FormControl, FormGroup, ValidationErrors } from '@angular/forms';
import { ClassConstructor, ClassTransformOptions } from 'class-transformer-global-storage';
import { ValidatorOptions } from 'class-validator-multi-lang';
import { BehaviorSubject, Observable, ReplaySubject, Subscription } from 'rxjs';
export declare type DeepPartial<T> = {
    [P in keyof T]?: DeepPartial<T[P]>;
};
export interface IDynamicControlMetadata {
    classType: ClassConstructor<unknown> | null;
    propertyName: string | null;
    propertyIndex?: number;
    isArray: boolean;
    properties: IDynamicControlMetadata[];
    withAncestors?: boolean;
    ignore?: boolean;
    parent?: IDynamicControlMetadata;
}
export interface DynamicControlOptions {
    rootValue: any;
    controlPath: string;
    control: AbstractControl | DynamicFormProperties;
    metadata: IDynamicControlMetadata;
}
export declare type DynamicFormProperties<T = unknown> = {
    classTransformMetadata: IDynamicControlMetadata;
    externalErrors: ClassValidatorErrors;
    classValidatorErrors: ClassValidatorErrors;
    externalErrorsSubject: BehaviorSubject<ClassValidatorErrors>;
    classValidatorErrorsSubject: BehaviorSubject<ClassValidatorErrors>;
    customValidateErrors: BehaviorSubject<ShortValidationErrors>;
    parent: AbstractControl['parent'] | DynamicFormProperties<T>;
    value: AbstractControl['value'];
};
export declare type DynamicFormArray = FormArray & DynamicFormProperties;
export declare type DynamicFormControl = FormControl & DynamicFormProperties;
export interface DynamicFormBuilderOptions {
    classValidatorOptions?: ValidatorOptions;
    classTransformOptions?: ClassTransformOptions;
    classTransformToPlainOptions?: ClassTransformOptions;
}
export interface ClassValidatorErrors {
    [key: string]: {
        messages?: string[];
        children?: ClassValidatorErrors;
    };
}
export interface ShortValidationErrors {
    [key: string]: string[] | ShortValidationErrors | any;
}
export declare type DynamicFormGroup<T = never, TJSON = T> = FormGroup & DynamicFormProperties<T> & {
    globalDynamicFormBuilderOptionsChangedSubscription: Subscription;
    dynamicFormBuilderOptions: DynamicFormBuilderOptions;
    commonAsyncValidatorSubject: ReplaySubject<null | string>;
    commonAsyncValidatorFirstChanged: boolean;
    commonAsyncValidatorFirstChangedSubscription: Subscription;
    commonAsyncValidator: (control: DynamicFormProperties | AbstractControl) => Observable<ValidationErrors | null>;
    setExternalErrors(externalErrors: ClassValidatorErrors): void;
    getExternalErrors(): ClassValidatorErrors;
    getClassValidatorErrors: (control: AbstractControl | null, nullIfEmpty: boolean) => Observable<ClassValidatorErrors[''] | null>;
    setDynamicValue(value: DeepPartial<T>, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    patchDynamicFormBuilderOptions(dynamicFormBuilderOptions: DynamicFormBuilderOptions): void;
    refresh: () => void;
    object: T;
    json: TJSON;
    getObject: () => T;
    setObject: (object: T) => void;
    getJSON: () => TJSON;
    setJSON: (object: TJSON) => void;
};
